      PROGRAM LANISO
c-------------------------------------------------------------------------
c                                "LANISO"                             
c                                                                         
c  This program computes the controlled-source tensor magnetotelluric 
c  (CSTMT) impendance tensor and tipper vector of a layered  half-space 
c  with azimuthal anisotropy.
c
c  The theory about this program is described in the article 
c  "Li, X. and Pedersen, L.B., 1992, Controlled-source tensor 
c  magnetotelluric responses of a layered earth with azimuthal 
c  anisotropy, Geophys. J. Int., 111, 91-103".
c  
c  Author: Xiaobo Li
c          Uppsala University
c          Department of Geophysics
c          Villavaegen 16
c          S-752 36 Uppsala
c          Sweden
c          Phone: +46 18 182373
c          Fax:   +46 18 501110
c          E-mail: XL@Geofys.UU.SE
c
c
c  Features of the program:
c  ........................
c  1. The x-axis is along the strike direction, the y-axis along the 
c     normal to strike direction, and the z-axis points downwards.
c  2. Electrical dipole sources are located at the origin.
c  3. In each layer there are three parameters except in the last layer 
c     in which there is no thickness. They are the conductivity along the 
c     x-axis, coefficient of anisotropy (square root of the ratio of 
c     the conductivity along the x-axis to the one along the y-axis) 
c     and thickness.
c  4. All quantities are given in SI units
c        E in V/m
c        H in A/m
c        Z (=E/H) in ohm
c        Conductivity in S/m
c        Resistivity in ohm/m
c        Frequency in Hz
c        Length in m
c  5. The time dependence is exp(-iwt).
c  6. This program was developed on SUN SPARC workstations.
c  
c-----------------------------------------------------------------------
c  Input file: parame.in
c  Output file: result.out
c
c---------------------------------------------------------------------- 
c
c  Last revision is on May 1, 1994
c-----------------------------------------------------------------------

      implicit real*8(a-h,o-z)
      real*8 x,y, freque
      complex*16 zab(6)
      external hand

      i1 = ieee_handler('set', 'division', hand)
      i2 = ieee_handler('set', 'underflow',hand)
      i3 = ieee_handler('set', 'overflow', hand)
      i4 = ieee_handler('set', 'invalid',  hand)

c-----Define coordinates of observation point
      x=1000.
      y=1000.

c-----Define frequency
      freque=1.   

      open(unit=4, file='result.out', status='UNKNOWN',
     *     access='SEQUENTIAL', form='FORMATTED' )

      call tensor( zab, x, y, freque )

      stop
      end

c----This function routine gives error message if one of following
c----running time errors occur
      integer function hand (sig, code, context )
      integer sig, code, context(5)
      character label*16
      if (loc(code) .eq. 208) label = 'invalid'
      if (loc(code) .eq. 200) label = 'division by zero'
      if (loc(code) .eq. 212) label = 'overflow'
      if (loc(code) .eq. 204) label = 'underflow'
c      if (loc(code) .eq. 196) label = 'inexact'
      write ( *, 77 ) loc(code), label, context(4)
  77  format('ieee exception code ', i3, ',' , a17, ',' , 'at pc', i5)
      stop
      end



      subroutine tensor(zab,xx,yy,freque)
c-----------------------------------------------------------------------
c    This subroutine calculates impedance tensor and tipper vector.
c
c        zab(1)=zxx,    zab(2)=zxy
c        zab(3)=zyx,    zab(4)=zyy
c        zab(5)=a  ,    zab(6)=b
c_______________________________________________________________________
      implicit real*8(a-h, o-z)
      complex*16 zab(6)
      complex*16  x_ex, x_ey, x_hx, x_hy, x_hz,
     *            y_ex, y_ey, y_hx, y_hy, y_hz
      complex*16 det
      x=xx
      y=yy
      freq=freque
      call  fields( x_ex, x_ey, x_hx, x_hy, x_hz,
     *              y_ex, y_ey, y_hx, y_hy, y_hz, x, y, freq )
      det= x_hx * y_hy - x_hy * y_hx
      zab(1)=( x_ex * y_hy - y_ex * x_hy )/det
      zab(2)=( y_ex * x_hx - x_ex * y_hx )/det
      zab(3)=( x_ey * y_hy - y_ey * x_hy )/det
      zab(4)=( y_ey * x_hx - x_ey * y_hx )/det
      zab(5)=( x_hz * y_hy - y_hz * x_hy )/det
      zab(6)=( y_hz * x_hx - x_hz * y_hx )/det

      write(4, 400) x_ex, x_ey, x_hx, x_hy, x_hz
      write(4, 500) y_ex, y_ey, y_hx, y_hy, y_hz
      write(4, 600) zab

      return
 400  format(//' fields by a x-directed dipole'//
     *'  Ex=',2e12.4,'  Ey=',2e12.4/
     *'  Hx=',2e12.4,'  Hy=',2e12.4/
     *'  Hz=',2e12.4,///)
 500  format(' fields by a y-directed dipole'//
     *'  Ex=',2e12.4,'  Ey=',2e12.4/
     *'  Hx=',2e12.4,'  Hy=',2e12.4/
     *'  Hz=',2e12.4,///)
 600  format(' impedance tensor and tipper vector'//
     *' Zxx=',2e12.4, ' Zxy=', 2e12.4/
     *' Zyx=',2e12.4, ' Zyy=', 2e12.4/
     *'   A=',2e12.4, '   B=', 2e12.4)  


      end

      subroutine fields(x_ex,x_ey,x_hx,x_hy,x_hz,
     *                  y_ex,y_ey,y_hx,y_hy,y_hz,xx,yy,freque)
c-----------------------------------------------------------------------
c  This subroutine calculates the surface fields by x-directed 
c  dipole and y-directed dipole on the surface. Source moments are
c  given as one.
c
c  Input data:
c    xx:  x coordinate of observation point
c    yy:  y coordinate of observation point
c    freque: frequency
c
c
c  Source dipoles are located at origin of the coordinate system. 
c
c  Output data:
c    x_ex: x component of the electric field by a x-directed dipole
c    x_ey: y component of the electric field by a x-directed dipole
c    x_hx: x component of the magnetic field by a x-directed dipole
c    x_hy: y component of the magnetic field by a x-directed dipole
c    x_hz: z component of the magnetic field by a x-directed dipole
c    y_ex: x component of the electric field by a y-directed dipole
c    y_ey: y component of the electric field by a y-directed dipole
c    y_hx: x component of the magnetic field by a y-directed dipole
c    y_hy: y component of the magnetic field by a y-directed dipole
c    y_hz: z component of the magnetic field by a y-directed dipole
c
c    The array 'slh' contains layer parameters which are read in from  
c    input file 'parame.in'
c    mcap: number of layer
c    slh( 3*(i-1)+1): conductivity in strike direction in layer i
c    slh( 3*(i-1)+2): anisotropy parameter of layer i
c    slh( 3*(i-1)+2): thickness of layer i
c       i = 1,2, ..., mcap
c------------------------------------------------------------------------
      implicit real*8(a-h,o-z)
      parameter(lp=180, ln=180, ibp=-130, ibn=-130, ndec=10)

      complex*16 x_ex,x_ey,x_hx,x_hy,x_hz,
     *           y_ex,y_ey,y_hx,y_hy,y_hz
      real hp,hn
      common /cop/hp(lp)
      common /con/hn(ln)
      common /parame/slh(100)
      common /freq/freq
      common /mcap/mcap
      common /blockk/kx,ky,kxs,kys,kappa,kappas 
      common /k0s/k0s,k0
      real*8 kx,ky,kxs,kys,kappa,kappas
      real*8 pi,epsiln
      complex*16 k0s,k0,u0,alpha0
      complex*16 six,siy,sixy
      complex*16 ap, x_axn, x_ayn, x_azn, y_ayn, y_azn

      complex*16 x_tthx, x_tthy, x_tthz, x_ttex, x_ttey
      complex*16 y_tthx, y_tthy, y_tthz, y_ttex, y_ttey
      complex*16 y_ex _nn, y_ex _np, y_ex _pn, y_ex _pp,
     *           y_ey _nn, y_ey _np, y_ey _pn, y_ey _pp,
     *           y_hx _nn, y_hx _np, y_hx _pn, y_hx _pp,
     *           y_hy _nn, y_hy _np, y_hy _pn, y_hy _pp,
     *           y_hz _nn, y_hz _np, y_hz _pn, y_hz _pp, 

     *           x_ex _nn, x_ex _np, x_ex _pn, x_ex _pp,
     *           x_ey _nn, x_ey _np, x_ey _pn, x_ey _pp,
     *           x_hx _nn, x_hx _np, x_hx _pn, x_hx _pp,
     *           x_hy _nn, x_hy _np, x_hy _pn, x_hy _pp,
     *           x_hz _nn, x_hz _np, x_hz _pn, x_hz _pp, 
    
     *           y_kex _nn, y_kex _np, y_kex _pn, y_kex _pp,
     *           y_key _nn, y_key _np, y_key _pn, y_key _pp,
     *           y_khx _nn, y_khx _np, y_khx _pn, y_khx _pp,
     *           y_khy _nn, y_khy _np, y_khy _pn, y_khy _pp,
     *           y_khz _nn, y_khz _np, y_khz _pn, y_khz _pp,

     *           x_kex _nn,x_kex _np,x_kex _pn,x_kex _pp,
     *           x_key _nn,x_key _np,x_key _pn,x_key _pp,
     *           x_khx _nn,x_khx _np,x_khx _pn,x_khx _pp,
     *           x_khy _nn,x_khy _np,x_khy _pn,x_khy _pp,
     *           x_khz _nn,x_khz _np,x_khz _pn,x_khz _pp 
    
      complex*16 y_ttex _nn, y_ttex _np, y_ttex _pn, y_ttex _pp,
     *           y_ttey _nn, y_ttey _np, y_ttey _pn, y_ttey _pp,
     *           y_tthx _nn, y_tthx _np, y_tthx _pn, y_tthx _pp,
     *           y_tthy _nn, y_tthy _np, y_tthy _pn, y_tthy _pp,
     *           y_tthz _nn, y_tthz _np, y_tthz _pn, y_tthz _pp, 

     *           x_ttex _nn, x_ttex _np, x_ttex _pn, x_ttex _pp,
     *           x_ttey _nn, x_ttey _np, x_ttey _pn, x_ttey _pp,
     *           x_tthx _nn, x_tthx _np, x_tthx _pn, x_tthx _pp,
     *           x_tthy _nn, x_tthy _np, x_tthy _pn, x_tthy _pp,
     *           x_tthz _nn, x_tthz _np, x_tthz _pn, x_tthz _pp 

      data pi/3.1415926535897932/, epsiln /8.854d-12/
      x=xx
      y=yy
      freq=freque

      open(unit=2, file='parame.in', status='old', access='sequential')

c----- read number of layer
      read(2,*) mcap

c----- read parameters of each layer
      do 10 i=1,mcap
         if(i.ne.mcap)then
            read(2,*)(slh(3*(i-1)+l),l=1,3)
         else
            read(2,*)(slh(3*(i-1)+l),l=1,2)
         end if
 10   continue
      close(2)

c----- write model parameters on screen to check
      write(*,4200) 
      do 20 i=1,mcap
         if(i.ne.mcap)then
            write(*,4300)i,(slh(3*(i-1)+l),l=1,3)
         else 
            write(*,4300)i,(slh(3*(i-1)+l),l=1,2)
         end if
 20   continue 
      write(4,4200) 
      do 25 i=1,mcap
         if(i.ne.mcap)then
            write(4,4300)i,(slh(3*(i-1)+l),l=1,3)
         else 
            write(4,4300)i,(slh(3*(i-1)+l),l=1,2)
         end if
 25   continue 

 4200 format(4x,'model parameters:'//
     *       4x,'layer      sigmat           lamda           thickness')
 4300 format(4x,i4,3f16.4) 

      write(4, 4400)x, y, freq
 4400 format(/'   coordinates of the observation point:',
     * '  x=', f7.1,'  y=', f7.1 //
     * '   frequency=', f7.1)

      k0s=16.d-7* pi**3 *epsiln*freq*freq
      k0=cdsqrt(k0s)
      alpha0=(0.0, -2.0)*pi*epsiln*freq 
      decn=1./dble(ndec)
      ax=dabs(x)
      ay=dabs(y)

      if ( ax .le. 1.d-2 .and. ay .le. 1.d-2 )then
         write(*,*)'stop! x and y are too small; check input data'
         write(*,*)'x=',x, '  y=',y
         stop
      end if 
      if ( ax .lt. 1.d-3  .or.  ay .lt. 1.d-3 ) then
         goto 300
      end if
      if ( ax/ay .gt. 10.  .or.  ay/ax .gt. 10.) then
         goto 300
      end if 

      rou=dsqrt(ax*ax+ay*ay) 
      reax=1./ax
      reay=1./ay 

      x_hx=(0.0, 0.0)
      x_hy=(0.0, 0.0)
      x_hz=(0.0, 0.0)
      x_ex=(0.0, 0.0) 
      x_ey=(0.0, 0.0)
      y_hx=(0.0, 0.0)
      y_hy=(0.0, 0.0)
      y_hz=(0.0, 0.0)
      y_ex=(0.0, 0.0) 
      y_ey=(0.0, 0.0)

      do 200 i=50,165
         tx=dble(-131+i)*decn
         kx=10.**tx*reax 
         kxs=kx*kx
         do 100 j=50,165
            ty=dble(-131+j)*decn
            ky=10.**ty*reay
            kys=ky*ky
            kappas=kxs+kys
            kappa=dsqrt(kappas)

            hpp=dble( hp(i) ) * dble( hp(j) )
            hnn=dble( hn(i) ) * dble( hn(j) )
            hpn=dble( hp(i) ) * dble( hn(j) ) 
            hnp=dble( hn(i) ) * dble( hp(j) )  
            u0=cdsqrt( kx*kx + ky*ky - k0s )
            akxy=dsqrt( kx*ky )
            call coeffi(ap, x_axn, x_ayn, x_azn, y_ayn, y_azn )

            x_tthx=(0.0, 1.0)*ky*x_azn - u0*x_ayn

            x_tthy=-u0*ap + u0*x_axn - (0.0, 1.0)*kx*x_azn 

            x_tthz=(0.0, 1.0)*( kx*x_ayn - ky*( ap + x_axn)  )

            x_ttex=k0s*(ap+x_axn) + (0.0, 1.0)*kx*

     *      ( (0.0, 1.0)*kx*(ap+x_axn)+ (0.0, 1.0)*ky*x_ayn + u0*x_azn) 

            x_ttey=k0s*x_ayn + (0.0, 1.0)*ky*

     *      ( (0.0, 1.0)*kx*(ap+x_axn)+ (0.0, 1.0)*ky*x_ayn + u0*x_azn)

            y_tthx=(0.0, 1.0)*ky*y_azn + u0*( ap -y_ayn )

            y_tthy= - (0.0, 1.0)*kx*y_azn

            y_tthz= (0.0, 1.0)*kx*( ap + y_ayn )

            y_ttex=(0.0, 1.0)*kx*( (0.0, 1.0)*ky*(ap+y_ayn)+u0*y_azn )
     
            y_ttey= k0s*( ap + y_ayn) +

     *             (0.0, 1.0)*ky*( (0.0, 1.0)*ky*(ap+y_ayn)+u0*y_azn )

            x_hx = x_hx + x_tthx*akxy*hpp
            x_hy = x_hy + x_tthy*akxy*hnn
            x_hz = x_hz + x_tthz*akxy*hnp
            x_ex = x_ex + x_ttex*akxy*hnn
            x_ey = x_ey + x_ttey*akxy*hpp
            y_hx = y_hx + y_tthx*akxy*hnn
            y_hy = y_hy + y_tthy*akxy*hpp
            y_hz = y_hz + y_tthz*akxy*hpn
            y_ex = y_ex + y_ttex*akxy*hpp
            y_ey = y_ey + y_ttey*akxy*hnn
 100     continue
 200  continue
      c=1./(2.*pi*dsqrt(ax*ay))
      six=(0.0,1.0)*sign(1.,x)
      siy=(0.0,1.0)*sign(1.,y)
 
      x_ex=        c*x_ex/alpha0
      x_ey=six*siy*c*x_ey/alpha0
      x_hx=six*siy*c*x_hx
      x_hy=        c*x_hy
      x_hz=    siy*c*x_hz
      y_ex=six*siy*c*y_ex/alpha0
      y_ey=        c*y_ey/alpha0
      y_hx=        c*y_hx
      y_hy=six*siy*c*y_hy
      y_hz=six*    c*y_hz
      go to 9999
c
c   abs(x/y) is less than 0.1  or greater than 10.
c
c   co=1/sqrt(2)=cos(pi/4)=sin(pi/4)
c

 300  co=0.70710678118654755
      px=co*( x+y)
      py=co*(-x+y)
      apx=dabs(px)
      apy=dabs(py)
      reax=1./apx
      reay=1./apy

      y_ex_nn=(0.0, 0.0)
      y_ex_np=(0.0, 0.0)
      y_ex_pn=(0.0, 0.0)
      y_ex_pp=(0.0, 0.0)

      y_ey_nn=(0.0, 0.0)
      y_ey_np=(0.0, 0.0)
      y_ey_pn=(0.0, 0.0)
      y_ey_pp=(0.0, 0.0)

      y_hx_nn=(0.0, 0.0)
      y_hx_np=(0.0, 0.0)
      y_hx_pn=(0.0, 0.0)
      y_hx_pp=(0.0, 0.0)

      y_hy_nn=(0.0, 0.0)
      y_hy_np=(0.0, 0.0)
      y_hy_pn=(0.0, 0.0)
      y_hy_pp=(0.0, 0.0)

      y_hz_nn=(0.0, 0.0)
      y_hz_np=(0.0, 0.0)
      y_hz_pn=(0.0, 0.0)
      y_hz_pp=(0.0, 0.0)

      x_ex_nn=(0.0, 0.0)
      x_ex_np=(0.0, 0.0)
      x_ex_pn=(0.0, 0.0)
      x_ex_pp=(0.0, 0.0)

      x_ey_nn=(0.0, 0.0)
      x_ey_np=(0.0, 0.0)
      x_ey_pn=(0.0, 0.0)
      x_ey_pp=(0.0, 0.0)

      x_hx_nn=(0.0, 0.0)
      x_hx_np=(0.0, 0.0)
      x_hx_pn=(0.0, 0.0)
      x_hx_pp=(0.0, 0.0)

      x_hy_nn=(0.0, 0.0)
      x_hy_np=(0.0, 0.0)
      x_hy_pn=(0.0, 0.0)
      x_hy_pp=(0.0, 0.0)

      x_hz_nn=(0.0, 0.0)
      x_hz_np=(0.0, 0.0)
      x_hz_pn=(0.0, 0.0)
      x_hz_pp=(0.0, 0.0)
      do 320 i=50,165
         tx=dble(-131+i)*decn
         pkx=10.**tx*reax 
         do 310 j=50,165
            ty=dble(-131+j)*decn
            pky=10.**ty*reay
            hpp=dble( hp(i) ) * dble( hp(j) )
            hnn=dble( hn(i) ) * dble( hn(j) )
            hpn=dble( hp(i) ) * dble( hn(j) )
            hnp=dble( hn(i) ) * dble( hp(j) )

            akxy=dsqrt(pkx*pky)
            call kernal( x_kex_nn,x_key_nn,x_khx_nn,x_khy_nn,x_khz_nn, 
     *                   y_kex_nn,y_key_nn,y_khx_nn,y_khy_nn,y_khz_nn, 
     *                  -pkx, -pky )
            call kernal( x_kex_np,x_key_np,x_khx_np,x_khy_np,x_khz_np,
     *                   y_kex_np,y_key_np,y_khx_np,y_khy_np,y_khz_np,
     *                  -pkx,  pky )
            call kernal( x_kex_pn,x_key_pn,x_khx_pn,x_khy_pn,x_khz_pn,
     *                   y_kex_pn,y_key_pn,y_khx_pn,y_khy_pn,y_khz_pn, 
     *                   pkx, -pky )
            call kernal( x_kex_pp,x_key_pp,x_khx_pp,x_khy_pp,x_khz_pp,
     *                   y_kex_pp,y_key_pp,y_khx_pp,y_khy_pp,y_khz_pp, 
     *                   pkx,  pky )

            y_ttex_nn=hnn*( y_kex_pp+y_kex_np+y_kex_pn+y_kex_nn )*akxy 
            y_ttex_np=hnp*( y_kex_pp+y_kex_np-y_kex_pn-y_kex_nn )*akxy 
            y_ttex_pn=hpn*( y_kex_pp-y_kex_np+y_kex_pn-y_kex_nn )*akxy
            y_ttex_pp=hpp*( y_kex_pp-y_kex_np-y_kex_pn+y_kex_nn )*akxy 

            y_ttey_nn=hnn*( y_key_pp+y_key_np+y_key_pn+y_key_nn )*akxy 
            y_ttey_np=hnp*( y_key_pp+y_key_np-y_key_pn-y_key_nn )*akxy 
            y_ttey_pn=hpn*( y_key_pp-y_key_np+y_key_pn-y_key_nn )*akxy
            y_ttey_pp=hpp*( y_key_pp-y_key_np-y_key_pn+y_key_nn )*akxy

            y_tthx_nn=hnn*( y_khx_pp+y_khx_np+y_khx_pn+y_khx_nn )*akxy 
            y_tthx_np=hnp*( y_khx_pp+y_khx_np-y_khx_pn-y_khx_nn )*akxy 
            y_tthx_pn=hpn*( y_khx_pp-y_khx_np+y_khx_pn-y_khx_nn )*akxy
            y_tthx_pp=hpp*( y_khx_pp-y_khx_np-y_khx_pn+y_khx_nn )*akxy

            y_tthy_nn=hnn*( y_khy_pp+y_khy_np+y_khy_pn+y_khy_nn )*akxy 
            y_tthy_np=hnp*( y_khy_pp+y_khy_np-y_khy_pn-y_khy_nn )*akxy 
            y_tthy_pn=hpn*( y_khy_pp-y_khy_np+y_khy_pn-y_khy_nn )*akxy
            y_tthy_pp=hpp*( y_khy_pp-y_khy_np-y_khy_pn+y_khy_nn )*akxy

            y_tthz_nn=hnn*( y_khz_pp+y_khz_np+y_khz_pn+y_khz_nn )*akxy 
            y_tthz_np=hnp*( y_khz_pp+y_khz_np-y_khz_pn-y_khz_nn )*akxy 
            y_tthz_pn=hpn*( y_khz_pp-y_khz_np+y_khz_pn-y_khz_nn )*akxy
            y_tthz_pp=hpp*( y_khz_pp-y_khz_np-y_khz_pn+y_khz_nn )*akxy

            x_ttex_nn=hnn*( x_kex_pp+x_kex_np+x_kex_pn+x_kex_nn )*akxy 
            x_ttex_np=hnp*( x_kex_pp+x_kex_np-x_kex_pn-x_kex_nn )*akxy 
            x_ttex_pn=hpn*( x_kex_pp-x_kex_np+x_kex_pn-x_kex_nn )*akxy
            x_ttex_pp=hpp*( x_kex_pp-x_kex_np-x_kex_pn+x_kex_nn )*akxy 

            x_ttey_nn=hnn*( x_key_pp+x_key_np+x_key_pn+x_key_nn )*akxy 
            x_ttey_np=hnp*( x_key_pp+x_key_np-x_key_pn-x_key_nn )*akxy 
            x_ttey_pn=hpn*( x_key_pp-x_key_np+x_key_pn-x_key_nn )*akxy
            x_ttey_pp=hpp*( x_key_pp-x_key_np-x_key_pn+x_key_nn )*akxy

            x_tthx_nn=hnn*( x_khx_pp+x_khx_np+x_khx_pn+x_khx_nn )*akxy 
            x_tthx_np=hnp*( x_khx_pp+x_khx_np-x_khx_pn-x_khx_nn )*akxy 
            x_tthx_pn=hpn*( x_khx_pp-x_khx_np+x_khx_pn-x_khx_nn )*akxy
            x_tthx_pp=hpp*( x_khx_pp-x_khx_np-x_khx_pn+x_khx_nn )*akxy

            x_tthy_nn=hnn*( x_khy_pp+x_khy_np+x_khy_pn+x_khy_nn )*akxy 
            x_tthy_np=hnp*( x_khy_pp+x_khy_np-x_khy_pn-x_khy_nn )*akxy 
            x_tthy_pn=hpn*( x_khy_pp-x_khy_np+x_khy_pn-x_khy_nn )*akxy
            x_tthy_pp=hpp*( x_khy_pp-x_khy_np-x_khy_pn+x_khy_nn )*akxy

            x_tthz_nn=hnn*( x_khz_pp+x_khz_np+x_khz_pn+x_khz_nn )*akxy 
            x_tthz_np=hnp*( x_khz_pp+x_khz_np-x_khz_pn-x_khz_nn )*akxy 
            x_tthz_pn=hpn*( x_khz_pp-x_khz_np+x_khz_pn-x_khz_nn )*akxy
            x_tthz_pp=hpp*( x_khz_pp-x_khz_np-x_khz_pn+x_khz_nn )*akxy

              
            y_ex_nn = y_ex_nn + y_ttex_nn
            y_ex_np = y_ex_np + y_ttex_np
            y_ex_pn = y_ex_pn + y_ttex_pn
            y_ex_pp = y_ex_pp + y_ttex_pp

            y_ey_nn = y_ey_nn + y_ttey_nn
            y_ey_np = y_ey_np + y_ttey_np
            y_ey_pn = y_ey_pn + y_ttey_pn
            y_ey_pp = y_ey_pp + y_ttey_pp

            y_hx_nn = y_hx_nn + y_tthx_nn
            y_hx_np = y_hx_np + y_tthx_np
            y_hx_pn = y_hx_pn + y_tthx_pn
            y_hx_pp = y_hx_pp + y_tthx_pp

            y_hy_nn = y_hy_nn + y_tthy_nn
            y_hy_np = y_hy_np + y_tthy_np
            y_hy_pn = y_hy_pn + y_tthy_pn
            y_hy_pp = y_hy_pp + y_tthy_pp

            y_hz_nn = y_hz_nn + y_tthz_nn
            y_hz_np = y_hz_np + y_tthz_np
            y_hz_pn = y_hz_pn + y_tthz_pn
            y_hz_pp = y_hz_pp + y_tthz_pp
            
            x_ex_nn = x_ex_nn + x_ttex_nn
            x_ex_np = x_ex_np + x_ttex_np
            x_ex_pn = x_ex_pn + x_ttex_pn
            x_ex_pp = x_ex_pp + x_ttex_pp

            x_ey_nn = x_ey_nn + x_ttey_nn
            x_ey_np = x_ey_np + x_ttey_np
            x_ey_pn = x_ey_pn + x_ttey_pn
            x_ey_pp = x_ey_pp + x_ttey_pp

            x_hx_nn = x_hx_nn + x_tthx_nn
            x_hx_np = x_hx_np + x_tthx_np
            x_hx_pn = x_hx_pn + x_tthx_pn
            x_hx_pp = x_hx_pp + x_tthx_pp

            x_hy_nn = x_hy_nn + x_tthy_nn
            x_hy_np = x_hy_np + x_tthy_np
            x_hy_pn = x_hy_pn + x_tthy_pn
            x_hy_pp = x_hy_pp + x_tthy_pp

            x_hz_nn = x_hz_nn + x_tthz_nn
            x_hz_np = x_hz_np + x_tthz_np
            x_hz_pn = x_hz_pn + x_tthz_pn
            x_hz_pp = x_hz_pp + x_tthz_pp
 310     continue
 320  continue
      six = (0.0, 1.0) * sign(1.0, px)
      siy = (0.0, 1.0) * sign(1.0, py)
      sixy= six*siy 
      y_ex = y_ex_nn + siy * y_ex_np + six * y_ex_pn + sixy * y_ex_pp
      y_ey = y_ey_nn + siy * y_ey_np + six * y_ey_pn + sixy * y_ey_pp
      y_hx = y_hx_nn + siy * y_hx_np + six * y_hx_pn + sixy * y_hx_pp
      y_hy = y_hy_nn + siy * y_hy_np + six * y_hy_pn + sixy * y_hy_pp
      y_hz = y_hz_nn + siy * y_hz_np + six * y_hz_pn + sixy * y_hz_pp

      x_ex = x_ex_nn + siy * x_ex_np + six * x_ex_pn + sixy * x_ex_pp
      x_ey = x_ey_nn + siy * x_ey_np + six * x_ey_pn + sixy * x_ey_pp
      x_hx = x_hx_nn + siy * x_hx_np + six * x_hx_pn + sixy * x_hx_pp
      x_hy = x_hy_nn + siy * x_hy_np + six * x_hy_pn + sixy * x_hy_pp
      x_hz = x_hz_nn + siy * x_hz_np + six * x_hz_pn + sixy * x_hz_pp

      c=1./( 8.*pi*dsqrt(apx*apy) )

      y_ex= c*y_ex/alpha0 
      y_ey= c*y_ey/alpha0
      y_hx= c*y_hx
      y_hy= c*y_hy
      y_hz= c*y_hz 

      x_ex= c*x_ex/alpha0 
      x_ey= c*x_ey/alpha0
      x_hx= c*x_hx
      x_hy= c*x_hy
      x_hz= c*x_hz 

 9999 return
      end
    

      subroutine kernal( x_kex, x_key, x_khx, x_khy, x_khz,
     *                   y_kex, y_key, y_khx, y_khy, y_khz, pkx, pky)
      implicit real*8 ( a-h, o-z )
      complex*16  x_kex, x_key, x_khx, x_khy, x_khz,
     *            y_kex, y_key, y_khx, y_khy, y_khz
      common /blockk/kx,ky,kxs,kys,kappa,kappas 
      common /k0s/k0s,k0
      real*8 kx,ky,kxs,kys,kappa,kappas 
      complex*16  ap, x_axn, x_ayn, x_azn, y_ayn, y_azn 
      complex*16 k0,k0s,u0
c
c----- co=1/sqrt(2.)=cos(pi/4)=sin(pi/4)
c
      co=0.70710678118654755
c
c----- coordinates are rotated by 45 degrees
c
      kx=co*(pkx-pky)
      ky=co*(pkx+pky)
      kxs=kx*kx
      kys=ky*ky
      kappas=kxs+kys
      kappa=dsqrt(kappas)
      u0=cdsqrt( kappas - k0s )
      call coeffi( ap, x_axn, x_ayn, x_azn, y_ayn, y_azn )
      x_khx=(0.0, 1.0)*ky*x_azn - u0*x_ayn

      x_khy=-u0*ap + u0*x_axn - (0.0, 1.0)*kx*x_azn 

      x_khz=(0.0, 1.0)*( kx*x_ayn - ky*( ap + x_axn)  )

      x_kex=k0s*(ap+x_axn) + (0.0, 1.0)*kx*

     *  ( (0.0, 1.0)*kx*(ap+x_axn)+ (0.0, 1.0)*ky*x_ayn + u0*x_azn) 

      x_key=k0s*x_ayn + (0.0, 1.0)*ky*

     *  ( (0.0, 1.0)*kx*(ap+x_axn)+ (0.0, 1.0)*ky*x_ayn + u0*x_azn)

      y_khx=(0.0, 1.0)*ky*y_azn + u0*( ap -y_ayn )

      y_khy= - (0.0, 1.0)*kx*y_azn

      y_khz= (0.0, 1.0)*kx*( ap + y_ayn )

      y_kex=(0.0, 1.0)*kx*( (0.0, 1.0)*ky*(ap+y_ayn)+u0*y_azn )
     
      y_key= k0s*( ap + y_ayn) +

     *  (0.0, 1.0)*ky*( (0.0, 1.0)*ky*(ap+y_ayn)+u0*y_azn )

      return
      end


c   
c---------------------------------------------------------------------------
c   this subroutine compute the reflection coffiecents
c
c---------------------------------------------------------------------------
      subroutine coeffi(ap, x_axn, x_ayn, x_azn, y_ayn, y_azn )
      implicit real*8(a-h,o-z)
      complex*16 ap,x_axn,x_ayn,x_azn,y_ayn,y_azn
      common /freq/freq
      common /mcap/mcap
      common /parame/slh(100)
      common /blockk/kx,ky,kxs,kys,kappa,kappas 
      common /k0s/k0s,k0
      complex*16 tfin(4,4),cinv(4),t(4,4),tinv(4,4),work(4,4),b(2)
      complex*16 x_tfin(6,6),x_cinv(6),x_t(6,6),x_tinv(6,6),
     *           x_work(6,6)      
      complex*16 x_tsub(3,3)
      complex*16 ap0x(3),am(3)
      complex*16 de,wcons
      complex*16 k0s,k0
      complex*16 axp,ayp
      real*8 kx,ky,kxs,kys,kappa,kappas 
      pi=3.1415926535897932

      mlayer=mcap
      if (mcap.gt.1)then      
         zbottm=0.
         do i=1,mcap-1
            zbottm=zbottm+slh(3*(i-1)+3)
            if (kappa*zbottm.gt.12.)then              
               mlayer=i
               goto 1000
            end if
         end do 
      end if

1000  call yproma(t,cinv,x_t,x_cinv, 0, 0)
      call minv(  t,  tfin,4,de)
      call minv(x_t,x_tfin,6,de)
   

      if (mlayer.eq.1)goto 500

c
c    compute (m-1) layer matries and multiply tf by them
c
      do 100 m=1,mlayer-1
         call yproma(t,cinv,x_t,x_cinv,m,1)

         call minv(  t,  tinv,4,de)
         call minv(x_t,x_tinv,6,de)
      
 

         call mulmat(tfin,t,work,4)

         call mulmat(x_tfin,x_t,x_work,6)

         do i=1,4
            do j=1,4
                 work(i,j)=  work(i,j)*  cinv(j)
            end do
         end do

         do i=1,6
            do j=1,6
               x_work(i,j)=x_work(i,j)*x_cinv(j)
            end do
         end do

          

         call mulmat(work,tinv,tfin,4)

         call mulmat(x_work,x_tinv,x_tfin,6) 
     
 100  continue
 500  call yproma(t,cinv,x_t,x_cinv,mlayer,0)     


 
      call mulmat(tfin,t,work,4)

      call mulmat(x_tfin,x_t,x_work,6)

      do i=1,4 
         do j=1,4
            tfin(i,j)=work(i,j)
         end do
      end do  

      do i=1,6
         do j=1,6
            x_tfin(i,j)=x_work(i,j)
         end do
      end do
 
      ayp=0.5/cdsqrt(kappas-k0s)
      axp=ayp
      ap=ayp


      wcons=tfin(1,1)*tfin(2,2)-tfin(1,2)*tfin(2,1)
      b(1)=ayp*tfin(2,2)/wcons 
      b(2)=-ayp*tfin(2,1)/wcons
      y_ayn=tfin(3,1)*b(1)+tfin(3,2)*b(2) 
      y_azn=tfin(4,1)*b(1)+tfin(4,2)*b(2) 


      do i=1,3
         do j=1,3  
            x_tsub(i,j)=x_tfin(i,j)
         end do
      end do

      ap0x(1)=axp
      ap0x(2)=( 0.0, 0.0 )
      ap0x(3)=( 0.0, 0.0 )
      call decomp(3, x_tsub)
      call solve(3, ap0x, am)
      do i=1,3
         wcons=( 0.0, 0.0 )
         do j=1,3
            wcons=wcons+x_tfin(3+i,j)*am(j)
         end do
         if      ( i .eq. 1 )then
              x_axn=wcons
         else if ( i. eq. 2 )then
              x_ayn=wcons
         else if ( i. eq. 3 )then
              x_azn=wcons
         end if
      end do
      return
      end

      subroutine summat(a,b,c,n)
c-----------------------------------------------------------------
c    addition of two matries c=a+b
c     input:  a      n*n matrix
c             b      n*n matrix
c             n      dimension of matries
c     output: c      n*n matrix    
c-----------------------------------------------------------------
      implicit real*8(a-h, o-z)
      complex*16 a(n,n), b(n,n), c(n,n)
      do 20 i=1,n
         do 10 j=1,n
            c(i,j)= a(i,j)+b(i,j)
 10      continue
 20   continue
      return
      end

      subroutine mulmat(a,b,c,n)
c----------------------------------------------------------------
c    multiplication of two matries c=a*b
c     input:  a       n*n matrix
c             b       n*n matrix
c             n       dimension of matries
c     output: c       n*n matrix
c----------------------------------------------------------------
      implicit real*8(a-h, o-z)
      complex*16 a(n,n), b(n,n), c(n,n)
      complex*16 work
      do 30 i=1,n
         do 20 j=1,n
            work=(0.0, 0.0)
            do 10 isum=1,n
               work=work+a(i,isum)*b(isum,j)
 10         continue
            c(i,j)=work
 20      continue
 30   continue
      return
      end



c--------------------------------------------------------------------------
c     cinv is computed if iden is  eaqual to 1
c     cinv is not computed if iden is eaqual to 0 
c--------------------------------------------------------------------------

      subroutine yproma(t,cinv,x_t,x_cinv,mm,iden)
      implicit real*8(a-h,o-z)
      complex*16 t(4,4),cinv(4),x_t(6,6),x_cinv(6)
      real*8 slh(100)
      real*8 kx,ky,kxs,kys,kappa,kappas
      complex*16 ks,u,v,ccc,u0,k0s,k0 
      complex*16 cccu, cccyu, cccyuu, cccxy, cccxyu,  cccx, cccy 
      complex*16 cdexpc
      common /parame/slh
      common /blockk/kx,ky,kxs,kys,kappa,kappas
      common /freq/freq
      common /mcap/mcap
      common /k0s/k0s,k0
      pi=3.1415926535897932
      m=mm
      if(m.eq.0)goto 100 
      ibefor=3*(m-1)
      sigmat=slh(ibefor+1)
      clam=slh(ibefor+2)
      h=slh(ibefor+3)
      ks=(0.,1.)*8*pi*pi*1.e-7*freq*sigmat
      u=cdsqrt(kappas-ks)
      v=cdsqrt(kxs+(kys-ks)/clam**2) 
      t(1,1)=1.     
      t(2,1)=0.
      t(3,1)=-v
      t(1,3)=1.
      t(2,3)=0.
      t(3,3)=v
      t(2,2)=1.
      t(2,4)=1.
      x_t(1,1)=1.0
      x_t(1,2)=0.0
      x_t(1,3)=0.0
      x_t(1,4)=1.0
      x_t(1,5)=0.0
      x_t(1,6)=0.0
      x_t(3,1)=0.0
      x_t(3,2)=0.0
      x_t(3,3)=1.0
      x_t(3,4)=0.0
      x_t(3,5)=0.0
      x_t(3,6)=1.0
      x_t(4,1)=-u
      x_t(4,2)=0.0
      x_t(4,3)=0.0
      x_t(4,4)=u
      x_t(4,5)=0.0
      x_t(4,6)=0.0
      x_t(2,2)=1.0
      x_t(2,5)=1.0
      x_t(5,2)=-v
      x_t(5,5)= v

      ccc=(0.,1.)*ky/ks
      t(4,1)=ccc
      t(4,3)=ccc
      x_t(6,2)=ccc
      x_t(6,5)=ccc

      ccc=1./(kys-ks)

      cccu=u*ccc
      t(4,2)= cccu
      t(4,4)=-cccu
      x_t(6,3)= cccu
      x_t(6,6)=-cccu 

      cccyu=(0.0, 1.0)*ky*cccu
      t(1,2)=-cccyu
      t(1,4)= cccyu
      x_t(2,3)=-cccyu
      x_t(2,6)= cccyu

      cccyuu=cccyu*u
      t(3,2)=cccyuu
      t(3,4)=cccyuu
      x_t(5,3)=cccyuu
      x_t(5,6)=cccyuu

      cccx=(0.0,-1.0)*kx*ccc 
      x_t(6,1)=cccx
      x_t(6,4)=cccx

      cccxy=(0.0, -1.0)*ky*cccx
      x_t(2,1)=cccxy
      x_t(2,4)=cccxy

      cccxyu=u*cccxy
      x_t(5,1)=-cccxyu
      x_t(5,4)= cccxyu
      if(iden .eq. 0)goto 10 

      ccc=cdexpc(v*h)
      cinv(1)=ccc
      x_cinv(2)=ccc

      ccc=1./ccc
      cinv(3)=ccc 
      x_cinv(5)=ccc

      ccc=cdexpc(u*h) 
      cinv(2)=ccc
      x_cinv(1)=ccc
      x_cinv(3)=ccc

      ccc=1./ccc
      cinv(4)=ccc
      x_cinv(4)=ccc
      x_cinv(6)=ccc

 10   return
c

 100  u0=cdsqrt(kappas-k0s)
      ccc=1./k0s 
      cccx=(0.,1.)*kx*ccc
      cccy=(0.,1.)*ky*ccc
      ccc=u0*ccc
      t(1,1)=1.
      t(1,2)=0.
      t(1,3)=1.
      t(1,4)=0.
      t(2,1)=0.
      t(2,2)=1.
      t(2,3)=0.
      t(2,4)=1.
      t(3,1)=-u0    
      t(3,2)=0.
      t(3,3)=u0    
      t(3,4)=0.
      t(4,1)=cccy
      t(4,2)=0.
      t(4,3)=cccy
      t(4,4)=ccc

      x_t(1,1)=1.0
      x_t(2,1)=0.0
      x_t(3,1)=0.0
      x_t(4,1)=-u0
      x_t(5,1)=0.0
      x_t(6,1)=cccx 
      x_t(1,2)=0.0
      x_t(2,2)=1.0
      x_t(3,2)=0.0
      x_t(4,2)=0.0
      x_t(5,2)=0.0
      x_t(6,2)=0.0
      x_t(1,3)=0.0
      x_t(2,3)=0.0
      x_t(3,3)=1.0
      x_t(4,3)=0.0
      x_t(5,3)=0.0
      x_t(6,3)=0.0           
      x_t(1,4)=1.0
      x_t(2,4)=0.0
      x_t(3,4)=0.0
      x_t(4,4)=u0
      x_t(5,4)=0.0
      x_t(6,4)=cccx
      x_t(1,5)=0.0
      x_t(2,5)=1.0
      x_t(3,5)=0.0
      x_t(4,5)=0.0
      x_t(5,5)=u0
      x_t(6,5)=cccy
      x_t(1,6)=0.0
      x_t(2,6)=0.0
      x_t(3,6)=1.0
      x_t(4,6)=0.0
      x_t(5,6)=0.0
      x_t(6,6)=ccc 
      return
      end

      complex*16 function cdexpc(z)
      implicit real*8(a-h, o-z)
      complex*16 z,z1     
      parameter( xl=100.)       
      z1=z                                                                  
      x1=dble(z1)                                                           
      if (dabs(x1).gt.xl) z1=dsign(xl,x1)                                    
      cdexpc=cdexp(z1)                                                  
      return                                                           
      end 


c     ..................................................................
c
c        subroutine minv
c
c        purpose
c           invert a matrix
c
c        usage
c           call minv(aori,a,n,de)
c
c        description of parameters
c           aori - input matrix
c           a -    resultant inverse.
c           n - order of matrix a
c           de- resultant determinant
c           bv - work vector of length n
c           cv - work vector of length n
c           mev- work vector of length n
c        remarks
c           matrix a must be a general matrix
c           if pivot element is less than 1.0e-8 the matrix
c           is taken for singular.this limit can be changed
c           by changing
c                            ep=1.0e-8
c           in the first executable statement to
c                            ep= the new value.
c
c        subroutines and function subprograms required
c           none
c
c        method
c           the standard gauss-jordan method is used. the determinant
c           is also calculated. a determinant of zero indicates that
c           the matrix is singular.
c
c                                                         
c     ..................................................................
c
      subroutine minv(aori,a,n,de)
      implicit real*8(a-h,o-z)
      dimension aori(n,n),a(n,n),mev(10),bv(10),cv(10)
      complex*16 a,bv,cv,aori 
      integer mev
      complex*16 de,y,w 
      ep=1.0e-15
      do 100 i=1,n
         do 100 j=1,n
            a(i,j)=aori(i,j)
 100  continue 
c
c        search for largest element
c
      de=1.0
      do 10 j=1,n
   10 mev(j)=j
      do 20 i=1,n
      y=0.0  
      do 30 j=i,n
      if(cdabs(a(i,j)).le.cdabs(y)) go to 30
      k=j        
      y=a(i,j)   
  30  continue    
      de=de*y  
      if(cdabs(y).lt.ep) then
      stop '4444'
      end if
c
c        interchange rows
c
      y=1./y                            
      do 40 j=1,n
      cv(j)=a(j,k)
      a(j,k)=a(j,i)
      a(j,i)=-cv(j)*y
      bv(j)=a(i,j)*y
   40 a(i,j)=a(i,j)*y
c
c        interchange columns
c
      a(i,i)=y
      j=mev(i)         
      mev(i)=mev(k)
      mev(k)=j     
      do 11 k=1,n 
      if (k.eq.i) go to 11
      do 12 j=1,n  
      if (j.eq.i) go to 12
      a(k,j)=a(k,j)-bv(j)*cv(k)
   12 continue
   11 continue
   20 continue
      do 33 i=1,n     
      do 44 k=1,n
      if(mev(k).eq.i) go to 55
   44 continue
   55 if (k.eq.i) go to 33
      do 66 j=1,n
      w=a(i,j)   
      a(i,j)=a(k,j)
   66 a(k,j)=w
      iw=mev(i)                   
      mev(i)=mev(k)  
      mev(k)=iw  
      de=-de      
   33 continue        
      return  
      end     
c
c

      block data h12p
      real hco12p(180),co1(76),co2(76),co3(28)
      common /cop/hco12p
      equivalence ( hco12p(1),  co1(1) ),( hco12p(77), co2(1) ),
     *            ( hco12p(153),co3(1) ) 

c       FAST HANKEL TRANSFORM COEFFICIENTS
c       PARAMETERS:
c          LENGTH: 180
c          STARTING POINT:-130
c          ANY: 0.5
c          NDEC: 10
c
      data co1/
     *  5.809722E-21,  8.206439E-21,  1.159193E-20,  1.637396E-20,
     *  2.312889E-20,  3.267051E-20,  4.614826E-20,  6.518608E-20,
     *  9.207766E-20,  1.300635E-19,  1.837193E-19,  2.595101E-19,
     *  3.665673E-19,  5.177914E-19,  7.314017E-19,  1.033131E-18,
     *  1.459334E-18,  2.061362E-18,  2.911759E-18,  4.112964E-18,
     *  5.809708E-18,  8.206421E-18,  1.159191E-17,  1.637405E-17,
     *  2.312893E-17,  3.267043E-17,  4.614816E-17,  6.518617E-17,
     *  9.207780E-17,  1.300632E-16,  1.837189E-16,  2.595105E-16,
     *  3.665692E-16,  5.177922E-16,  7.313999E-16,  1.033129E-15,
     *  1.459337E-15,  2.061366E-15,  2.911752E-15,  4.112962E-15,
     *  5.809717E-15,  8.206451E-15,  1.159192E-14,  1.637404E-14,
     *  2.312886E-14,  3.267050E-14,  4.614812E-14,  6.518605E-14,
     *  9.207773E-14,  1.300634E-13,  1.837194E-13,  2.595111E-13,
     *  3.665682E-13,  5.177914E-13,  7.314005E-13,  1.033129E-12,
     *  1.459332E-12,  2.061367E-12,  2.911753E-12,  4.112975E-12,
     *  5.809717E-12,  8.206462E-12,  1.159187E-11,  1.637404E-11,
     *  2.312885E-11,  3.267057E-11,  4.614806E-11,  6.518636E-11,
     *  9.207735E-11,  1.300639E-10,  1.837185E-10,  2.595118E-10,
     *  3.665666E-10,  5.177946E-10,  7.313934E-10,  1.033140E-09/
      data co2/
     *  1.459321E-09,  2.061392E-09,  2.911712E-09,  4.113028E-09,
     *  5.809616E-09,  8.206611E-09,  1.159165E-08,  1.637438E-08,
     *  2.312829E-08,  3.267142E-08,  4.614676E-08,  6.518846E-08,
     *  9.207395E-08,  1.300695E-07,  1.837099E-07,  2.595255E-07,
     *  3.665440E-07,  5.178299E-07,  7.313398E-07,  1.033225E-06,
     *  1.459181E-06,  2.061606E-06,  2.911374E-06,  4.113576E-06,
     *  5.808748E-06,  8.207963E-06,  1.158949E-05,  1.637784E-05,
     *  2.312277E-05,  3.267998E-05,  4.613285E-05,  6.521004E-05,
     *  9.203899E-05,  1.301226E-04,  1.836200E-04,  2.596563E-04,
     *  3.663117E-04,  5.181396E-04,  7.307156E-04,  1.033920E-03,
     *  1.457426E-03,  2.062917E-03,  2.905991E-03,  4.114711E-03,
     *  5.790420E-03,  8.200042E-03,  1.151926E-02,  1.630388E-02,
     *  2.282574E-02,  3.222489E-02,  4.478629E-02,  6.273282E-02,
     *  8.570576E-02,  1.174182E-01,  1.536326E-01,  1.977176E-01,
     *  2.288495E-01,  2.403111E-01,  1.654099E-01,  2.848540E-03,
     * -2.880154E-01, -3.690975E-01, -2.501130E-02,  5.718128E-01,
     * -3.922576E-01,  7.632519E-02,  5.162452E-02, -6.480095E-02,
     *  4.890481E-02, -3.269253E-02,  2.105518E-02, -1.338703E-02,
     *  8.472098E-03, -5.351488E-03,  3.378005E-03, -2.131753E-03/
      data co3/
     *  1.345134E-03, -8.487403E-04,  5.355186E-04, -3.379158E-04,
     *  2.132082E-04, -1.345237E-04,  8.487856E-05, -5.355410E-05,
     *  3.378994E-05, -2.131996E-05,  1.345182E-05, -8.487502E-06,
     *  5.355593E-06, -3.379109E-06,  2.132069E-06, -1.345228E-06,
     *  8.487801E-07, -5.355426E-07,  3.379004E-07, -2.131981E-07,
     *  1.345186E-07, -8.488092E-08,  5.355610E-08, -3.379120E-08,
     *  2.132054E-08, -1.345219E-08,  8.487818E-09, -5.355385E-09/
        end


c

      block data
      real hco12n(180),co1(76),co2(76),co3(28)
      common /con/hco12n
      equivalence (hco12n(1),co1(1)),(hco12n(77),co2(1)),
     *           (hco12n(153),co3(1))
c
c      fast hankel transform coefficients
c      parameters:
c         length: 180
c         starting point:-130
c         any: -.5
c         ndec: 10
c
       
      data co1/
     * 5.8097235e-08, 6.5186185e-08, 7.3140114e-08, 8.2064574e-08,
     * 9.2077997e-08, 1.0331304e-07, 1.1591915e-07, 1.3006346e-07,
     * 1.4593360e-07, 1.6374028e-07, 1.8371961e-07, 2.0613689e-07,
     * 2.3128942e-07, 2.5951107e-07, 2.9117570e-07, 3.2670454e-07,
     * 3.6656871e-07, 4.1129681e-07, 4.6148273e-07, 5.1779227e-07,
     * 5.8097271e-07, 6.5186214e-07, 7.3140154e-07, 8.2064457e-07,
     * 9.2078039e-07, 1.0331310e-06, 1.1591922e-06, 1.3006354e-06,
     * 1.4593370e-06, 1.6374033e-06, 1.8371975e-06, 2.0613697e-06,
     * 2.3128905e-06, 2.5951117e-06, 2.9117587e-06, 3.2670473e-06,
     * 3.6656879e-06, 4.1129706e-06, 4.6148307e-06, 5.1779161e-06,
     * 5.8097294e-06, 6.5186132e-06, 7.3140200e-06, 8.2064507e-06,
     * 9.2077917e-06, 1.0331312e-05, 1.1591928e-05, 1.3006360e-05,
     * 1.4593377e-05, 1.6374041e-05, 1.8371949e-05, 2.0613708e-05,
     * 2.3128923e-05, 2.5951082e-05, 2.9117597e-05, 3.2670490e-05,
     * 3.6656900e-05, 4.1129730e-05, 4.6148329e-05, 5.1779185e-05,
     * 5.8097314e-05, 6.5186221e-05, 7.3140094e-05, 8.2064536e-05,
     * 9.2077971e-05, 1.0331320e-04, 1.1591935e-04, 1.3006353e-04,
     * 1.4593371e-04, 1.6374033e-04, 1.8371956e-04, 2.0613678e-04,
     * 2.3128933e-04, 2.5951094e-04, 2.9117614e-04, 3.2670505e-04/
      data co2/
     * 3.6656915e-04, 4.1129749e-04, 4.6148256e-04, 5.1779207e-04,
     * 5.8097241e-04, 6.5186177e-04, 7.3140126e-04, 8.2064583e-04,
     * 9.2078000e-04, 1.0331324e-03, 1.1591939e-03, 1.3006346e-03,
     * 1.4593364e-03, 1.6374029e-03, 1.8371968e-03, 2.0613689e-03,
     * 2.3128940e-03, 2.5951103e-03, 2.9117633e-03, 3.2670526e-03,
     * 3.6656931e-03, 4.1129682e-03, 4.6148282e-03, 5.1779216e-03,
     * 5.8097253e-03, 6.5186154e-03, 7.3140087e-03, 8.2064401e-03,
     * 9.2077851e-03, 1.0331264e-02, 1.1591848e-02, 1.3006156e-02,
     * 1.4593132e-02, 1.6373441e-02, 1.8371176e-02, 2.0611871e-02,
     * 2.3126356e-02, 2.5945466e-02, 2.9109236e-02, 3.2652907e-02,
     * 3.6629796e-02, 4.1074961e-02, 4.6061382e-02, 5.1608227e-02,
     * 5.7819396e-02, 6.4650819e-02, 7.2254494e-02, 8.0387428e-02,
     * 8.9266151e-02, 9.8067060e-02, 1.0704951e-01, 1.1375766e-01,
     * 1.1832727e-01, 1.1396514e-01, 1.0049784e-01, 6.1295748e-02,
     *-1.6151229e-04,-1.1178777e-01,-2.2753608e-01,-3.3900315e-01,
     *-2.2513050e-01, 8.9826077e-02, 5.1251173e-01,-1.3199112e-01,
     *-3.3515251e-01, 3.6486661e-01,-2.3403901e-01, 1.3208568e-01,
     *-7.5674146e-02, 4.5230597e-02,-2.7828883e-02, 1.7371364e-02,
     *-1.0913696e-02, 6.8750028e-03,-4.3341480e-03, 2.7338061e-03/
      data co3/
     *-1.7247917e-03, 1.0881845e-03,-6.8660942e-04, 4.3320860e-04,
     *-2.7332612e-04, 1.7246284e-04,-1.0881423e-04, 6.8659472e-05,
     *-4.3320244e-05, 2.7334157e-05,-1.7246104e-05, 1.0881928e-05,
     *-6.8658810e-06, 4.3319305e-06,-2.7333572e-06, 1.7245934e-06,
     *-1.0881822e-06, 6.8657346e-07,-4.3321842e-07, 2.7333311e-07,
     *-1.7246745e-07, 1.0881719e-07,-6.8656675e-08, 4.3321421e-08,
     *-2.7333044e-08, 1.7246780e-08,-1.0881614e-08, 6.8659869e-09/
      end
c
c

      subroutine decomp(nn,a)
      implicit real*8(a-h,o-z)
      dimension a(nn,nn), ul(30,30), scales(300),ips(300)
      complex*16 a, ul, pivot,em
      common /dsi/ips
      common /ul/ul
      n=nn
c
      do 5 i=1,n
         ips(i)=i
         rownrm=0.0
         do 2 j=1,n
            ul(i,j)=a(i,j)
            if(rownrm-cdabs(ul(i,j))) 1,2,2
  1         rownrm=cdabs(ul(i,j))
  2      continue
         if(rownrm) 3,4,3
  3      scales(i)=1.0/rownrm
         goto 5
  4      call sing(1)
         scales(i)=0.0
  5   continue
c
      nm1=n-1
      do 17 k=1,nm1
      big=0.0
      do 11 i=k,n
         ip=ips(i)
         size=cdabs(ul(ip,k))*scales(ip)
         if(size-big)11,11,10
 10        big=size
           idxpiv=i
 11   continue
      if(big)13,12,13
 12          call sing(2)
             goto 17
 13      if(idxpiv-k)14,15,14
 14        j=ips(k)
           ips(k)=ips(idxpiv)
           ips(idxpiv)=j
 15      kp=ips(k)
         pivot=ul(kp,k)
          kp1=k+1
          do 16 i=kp1,n
             ip=ips(i)
             em=-ul(ip,k)/pivot
             ul(ip,k)=-em
             do 16 j=kp1,n
             ul(ip,j)=ul(ip,j)+em*ul(kp,j)
 16       continue
 17   continue
      kp=ips(n)
      if(cdabs(ul(kp,n)))19,18,19
 18   call sing(2)
 19   return
      end
c
c
      subroutine solve(nn,b,x)
      implicit real*8(a-h, o-z)
      dimension ul(30,30),b(nn),x(nn),ips(300)
      complex*16 ul,b,x,sum
      common /dsi/ips
      common /ul/ul
      n=nn
      np1=n+1
c
      ip=ips(1)
      x(1)=b(ip)
      do 2 i=2,n
         ip=ips(i)
         im1=i-1
         sum=(0.,0.)
         do 1 j=1,im1
 1         sum=sum+ul(ip,j)*x(j)
 2    x(i)=b(ip)-sum
c
      ip=ips(n)
      x(n)=x(n)/ul(ip,n)
      do 4 iback=2,n
         i=np1-iback
c       
c     i goes    (n-1),...,1
c
          ip=ips(i)
          ip1=i+1
          sum=0.0
          do 3 j=ip1,n
 3           sum=sum+ul(ip,j)*x(j)
 4    x(i)=(x(i)-sum)/ul(ip,i)
         return
         end

      subroutine impruv (nn, a, b, x, digits)
      implicit real*8(a-h,o-z)
      complex*16 a(nn,nn),ul(30,30),b(nn),x(nn),r(30),dx(30)
c     uses abs(), amax1(), alog10()
      complex*16 sum,t
      common /ul/ul
      n=nn
c
      eps=1.0e-8
      itmax=20
c     *** eps and itmax are machine dependent. ***
c
      xnorm=0.0
      do 1 i=1,n
   1     xnorm=dmax1(xnorm,cdabs(x(i) ) )
      if (xnorm) 3, 2, 3
   2     digits = -dlog10(eps)
         go to 10
c
   3  do 9 iter = 1, itmax
         do 5 i =1,n
            sum=0.0
            do 4 j = 1, n
   4           sum = sum + a(i,j)*x(j)
            sum = b(i) - sum
   5     r(i) = sum

         call solve (n,r,dx)
         dxnorm = 0.0
         do 6 i =1, n
            t = x(i)
            x(i)=x(i)+dx(i)
            dxnorm = dmax1(dxnorm,cdabs(x(i)-t))
   6     continue
         if ( iter-1) 8, 7, 8
   7        digits = -dlog10(dmax1( dxnorm/xnorm, eps))
   8     if (dxnorm -eps*xnorm) 10, 10, 9
   9  continue
c     iteration did not converge
      call sing(3)
  10  return
      end 

c
      subroutine sing(iwhy)
 11   format(' matrix with zero row in decompose.')
 12   format(' singular matrix decompose. zero divide in solve.')
 13   format(' no convergence in impruv. matrix is nearly singular.')
      goto(1,2,3),iwhy
 1    write(*,11)
      goto 10
 2    write(*,12)
      goto 10
 3    write(*,13)
 10   return
      end
c
